==> Mudlib-CD.00.28.tar.gz: lib/cmd/std/tracer_tool_base.c <==
/*
 * tracer_tool_base.c
 * 
 * The basic tracer tool functions used in the commons soul for
 * parsing arguments to certain functions.
 *
 */

static object parse_list(string str);
static object find_item(object prev, string str);
static void assign(string var, mixed val);

#define TRACER_STORES	"_tracer_stores"
#define TRACER_VARS	"_tracer_vars"

static object
parse_list(string str)
{
    string          tmp,
                    rest;
    object          prev;

    prev = environment(this_interactive());
    while (prev && str)
    {
	if (sscanf(str, "%s:%s", tmp, rest) == 2)
	{
	    prev = find_item(prev, tmp);
	    str = rest;
	    continue;
	}
	prev = find_item(prev, str);
	break;
    }
    assign("$", prev);
    if (objectp(prev))
	write(file_name(prev) + "\n");
    return prev;
}

static object
find_item(object prev, string str)
{
    object ob, *ob_list;
    string tmp;
    int  tmptmp, i;
    object *u;

    if ((str == "here") || (str == "!"))
	return environment(this_interactive());
    if (str == "me") 
	return this_interactive();
    if (str == "^")
	return environment(prev);
    if (sscanf(str, "@%s", tmp) == 1)
	return find_living(tmp);
    if (sscanf(str, "*%s", tmp) == 1)
	return find_player(tmp);
    if (sscanf(str, "/%s", tmp) == 1)
    {
	call_other(tmp, "??");	/* Force load */
	return find_object(tmp);
    }
    if (sscanf(str, "$%d", tmptmp) == 1)
    {
	u = users();
	write("size: " + sizeof(u) + "\n");
	if (tmptmp >= sizeof(u) || tmptmp < 0)
	    return 0;
	return u[tmptmp - 1];
    }
    if (prev == 0)
	prev = environment(this_interactive());
    if (sscanf(str, "\"%s\"", tmp) == 1)
    {
	ob_list = all_inventory(prev);
	for (i = 0 ; i < sizeof(ob_list) ; i++)
	    if ((string)ob_list[i]->short() == tmp)
		return ob_list[i];
    }
    if (sscanf(str, "#%d", tmptmp) == 1)
    {
	if (prev == 0)
	    return 0;
	ob_list = all_inventory(prev);
	if (tmptmp > sizeof(ob_list))
	    return 0;

	return ob_list[tmptmp - 1];
    }
    return present(str, prev);
}

void
assign(string var, mixed val)
{
    int i;
    mixed *vars, *stores;

    vars = (mixed *)this_interactive()->query_prop(TRACER_VARS);
    stores = (mixed *)this_interactive()->query_prop(TRACER_STORES);

    while (i < sizeof(vars))
    {
	if (vars[i] == var)
	{
	    stores[i] = val;
	    return;
	}
	if (vars[i] == 0)
	{
	    vars[i] = var;
	    stores[i] = val;
	    return;
	}
	i += 1;
    }
    this_interactive()->add_prop(TRACER_VARS, vars);
    this_interactive()->add_prop(TRACER_STORES, stores);
}

==> Mudlib-CD.00.30.tar.gz: lib/cmd/std/tracer_tool_base.c <==
/*
 * tracer_tool_base.c
 * 
 * The basic tracer tool functions used in the commons soul for
 * parsing arguments to certain functions.
 *
 */

#pragma save_binary

inherit "/cmd/std/command_driver";
#include <filepath.h>

object parse_list(string str);
object find_item(object prev, string str);
void assign(string var, mixed val);

#define TRACER_STORES	"_tracer_stores"
#define TRACER_VARS	"_tracer_vars"

object
parse_list(string str)
{
    string          tmp,
                    rest;
    object          prev;

    prev = environment(this_interactive());
    while (prev && str)
    {
	if (sscanf(str, "%s:%s", tmp, rest) == 2)
	{
	    prev = find_item(prev, tmp);
	    str = rest;
	    continue;
	}
	prev = find_item(prev, str);
	break;
    }
    assign("$", prev);
    if (objectp(prev))
	write(file_name(prev) + "\n");
    return prev;
}

object
find_item(object prev, string str)
{
    object ob, *ob_list;
    string tmp, path;
    int  tmptmp, i;
    object *u;

    if ((str == "here") || (str == "!"))
	return environment(this_interactive());
    if (str == "me") 
	return this_interactive();
    if (str == "^")
	return environment(prev);
    if (sscanf(str, "@%s", tmp) == 1)
	return find_living(tmp);
    if (sscanf(str, "*%s", tmp) == 1)
	return find_player(tmp);
    if (sscanf(str, "$%d", tmptmp) == 1)
    {
	u = users();
	write("size: " + sizeof(u) + "\n");
	if (tmptmp >= sizeof(u) || tmptmp < 0)
	    return 0;
	return u[tmptmp - 1];
    }
    if (prev == 0)
	prev = environment(this_interactive());
    if (sscanf(str, "\"%s\"", tmp) == 1)
    {
	ob_list = all_inventory(prev);
	for (i = 0 ; i < sizeof(ob_list) ; i++)
	    if ((string)ob_list[i]->short() == tmp)
		return ob_list[i];
    }
    if (sscanf(str, "#%d", tmptmp) == 1)
    {
	if (prev == 0)
	    return 0;
	ob_list = all_inventory(prev);
	if (tmptmp > sizeof(ob_list))
	    return 0;

	return ob_list[tmptmp - 1];
    }
    if (ob = present(str, prev))
	return ob;
    if (ob = present(str, this_interactive()))
	return ob;

    path = FTPATH(this_interactive()->query_path() + "/", str);
    if (strlen(path))
    {
	catch(call_other(path, "??"));	/* Force load */
	return find_object(path);
    }
    return 0;
}

void
assign(string var, mixed val)
{
    int i;
    mixed *vars, *stores;

    vars = (mixed *)this_interactive()->query_prop(TRACER_VARS);
    stores = (mixed *)this_interactive()->query_prop(TRACER_STORES);

    while (i < sizeof(vars))
    {
	if (vars[i] == var)
	{
	    stores[i] = val;
	    return;
	}
	if (vars[i] == 0)
	{
	    vars[i] = var;
	    stores[i] = val;
	    return;
	}
	i += 1;
    }
    this_interactive()->add_prop(TRACER_VARS, vars);
    this_interactive()->add_prop(TRACER_STORES, stores);
}
